// BoostC version : 6.95
// Single user Full License for 1 node
// Target         : PIC18F4420
// Adappted to use with the nRF24L01
// by Martin Hug 2012
//
// The transmitter UNIT scans the analog/ digital i/o of the PIC18F4420
// and send the data using the spi to the nRF124L01.
////////////////////////////////////////////////////////////////////////////

#include <system.h>
#include "global.h";
#include "NG_TX_DRUM_V1.h"
#include "serial.h"
#include "nrf24L01.h"


#include <PIC18f4420.h>

#ifdef _PIC18
#pragma DATA _CONFIG1H, _OSC_INTIO67_1H     // Int OSC, Port RA6,RA7 I/O.
//#pragma DATA _CONFIG2H, _OSC_HSPLL_1H			// PLL active for external clock.        
#pragma DATA _CONFIG3H, _MCLRE_OFF_3H	
#endif _PIC18

#pragma CLOCK_FREQ 8000000                  // CLK 8Mhz.

#define PW_LED latd.5               				// PW LED.
#define PW_nRF latd.4               				// power up nRF24 (NOT NEEDED)

void main()
{
	init();                                   // device and peripheral initialisation.                     
  unsigned char get_byte;
  char count = 0, sel_array = 0;														
  short adc;
 
  nrf_tx_mode();                            // nRF transmitt mode & power up.
  nrf_reset_status();
  
	pad_flag_0 = 0x00;											
	off_flag_0 = 0x00;
 
  while (1){
		
    count = scan_analog(count);							// scan analog inputs and puts the results into a buffer.
 
			if ((pad_flag_0) && (!off_flag_0)){		// Trash-Hold of analog input.
		    nrf_put_byte(sel_array);						// send data wireless with nRF24L01.
      		while (nrf_irq_jack());						// waits until data has been sent.
	       	nrf_reset_status();
					pad_flag_0 = false;								 
					off_flag_0 = true;	 							// waits for adc = 0x00.
		  		}

			if ((pad_flag_1) && (!off_flag_1)){		// Trash-Hold of analog input.
		    nrf_put_byte(sel_array);						// send data wireless with nRF24L01.
      		while (nrf_irq_jack());						// waits until data has been sent.
        	nrf_reset_status();
					pad_flag_1 = false;								 
					off_flag_1 = true;	 							// waits for adc = 0x00.
		  		}
   	}
}
//
//

void init(){

  char aux = 0x00;
  osccon = 01110000b;                       // internal oscillator at 8 MHz.
  while (!iofs);                            // iofs OSCCON stable bit.

  trisd = 00001000b;                        // PORTD,3 input is connected to IRQ nRF24L01 (active low)
  portd = 0x00;                             // change this if needed. (loock also at nrf25L01.h).
  trisa = 00111111b;
	trise = 00000111b;												// RE0 - RE2 analog input.

	adcon1 |= 0x0F;                           // All digital IO's, no analog ports required.
  PW_LED = true;
  PW_nRF = 1;		                            // power for nRF24 (NOT NEEDED).

  adcon1 = 00011001b;                       // RA0 - RA6 analog, rest digital I/O, Vref active.
	adcon2 = 10101001b;                       // Result is right justified, conversion Ckl Fosc/8, TAD 12,
  //adcon2 = 10101100b;												// Result is right justified, conversion Ckl Fosc/4, TAD 12,


	  // Serial interface
	  serial_init(34);                        // (34) 57600 baud; DEBUG before BRG = 103 (19200 baud at 8MHz).
	  serial_printf("initializing serial port");
	  serial_print_lf();
	
	  // Wireless interface
	  nrf_init();
	  serial_printf("nRF24L01 config done...");
	  serial_print_lf();
	 return;
}

void PW_LED_OnOff()
{
	PW_LED = ~PW_LED;                         // toggle PW_LED (connected to PORTD,5).
}